SystemOrganization addCategory: #'UCA_IS2-Stack-Tests'!
SystemOrganization addCategory: #'UCA_IS2-Stack'!

Object subclass: #Stack
	instanceVariableNames: 'tamanio valor nodoSiguiente nodoAnterior'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UCA_IS2-Stack'!

!Stack commentStamp: 'dc 7/24/2005 15:41' prior: 0!
I implement a simple Stack. #push: adds a new object of any kind on top of the stack. #pop returns the first element and remove it from the stack. #top answer the first element of the stack without removing it.!

!Stack methodsFor: 'removing' stamp: 'Lucas 10/23/2012 14:51'!
anterior:  anterior
	nodoAnterior:=anterior! !

!Stack methodsFor: 'removing' stamp: 'Lucas 10/23/2012 14:49'!
isEmpty 
	^(valor =nil)! !

!Stack methodsFor: 'removing' stamp: 'Lucas 10/23/2012 14:47'!
pop
	"Returns the first element and remove it from the stack."

	| tmp |
	(nodoSiguiente =nil)   "para ""el nodo siguente tiene q apuntar a nil"
		ifFalse:  [^(nodoSiguiente pop) ] 
		ifTrue:   [ (nodoAnterior=nil)
					ifFalse: [nodoAnterior siguiente:nil.
							tmp:=valor.
							valor:=nil.
							^tmp]
					ifTrue: [tmp:=valor.
							valor:=nil.
							^tmp]
			]
	
	! !

!Stack methodsFor: 'adding' stamp: 'Lucas 10/23/2012 14:40'!
push: unNodo
	"Adds a new object of any kind on top of the stack."

	(valor = nil)  
		ifFalse:  [	nodoSiguiente:= Stack new.
					nodoSiguiente anterior: self.
				^(nodoSiguiente push: unNodo) ] 
		ifTrue:   [	valor:=unNodo.

]
	

	! !

!Stack methodsFor: 'removing' stamp: 'Lucas 10/23/2012 14:50'!
siguiente: siguiente
	nodoSiguiente:= siguiente! !

!Stack methodsFor: 'removing' stamp: 'Lucas 10/23/2012 14:49'!
size
	(nodoSiguiente=nil)  
		ifFalse:  [^((nodoSiguiente size)+1) ] 
		ifTrue:   [	(valor =nil)  
		ifFalse:  [^1 ] 
		ifTrue:   [^0]]
	! !

!Stack methodsFor: 'accessing' stamp: 'Lucas 10/23/2012 14:43'!
top
	"Answer the first element of the stack without removing it."
	(nodoSiguiente =nil)  
		ifFalse:  [^(nodoSiguiente top) ] 
		ifTrue:   [^valor]
	
	! !

TestCase subclass: #CollectionStackTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UCA_IS2-Stack-Tests'!

!CollectionStackTests methodsFor: 'examples' stamp: 'Lucas 10/23/2012 13:43'!
emptyStack

	"Deberia devolver un stack vaco. -jmaestri"
	
	^(Stack new)! !

!CollectionStackTests methodsFor: 'examples' stamp: 'Lucas 10/23/2012 13:42'!
nonEmptyStack
	|aStack|
	"Deberia devolver un stack no vaco. -jmaestri"

	
	^(Stack new
	add: 'No Vacio';
      yourself).! !

!CollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:37'!
testEmptyError

	| aStack |
	
	aStack := self emptyStack.
	
	self should: [ aStack top ] raise: Error.
	self should: [ aStack pop] raise: Error.
	
	aStack push: 'element'.
	
	self shouldnt: [ aStack top ] raise: Error.
	self shouldnt: [ aStack pop] raise: Error.
	
	"The stack is empty again due to previous pop"
	self should: [ aStack top ] raise: Error.
	self should: [ aStack pop] raise: Error.! !

!CollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:19'!
testIsEmpty

	self assert: (self emptyStack isEmpty).
	self deny: (self nonEmptyStack isEmpty).! !

!CollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:37'!
testPop

	| aStack result element |

	element := 'anElement'.
	aStack := self emptyStack.
	
	aStack push: 'a'.
	aStack push: element.
	
	result := aStack pop.
	self assert: result = element.
	self assert: result == element.
	self assert: aStack size = 1.
	
	aStack pop.
	self assert: aStack isEmpty! !

!CollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:20'!
testPush

	| aStack |

	aStack := self emptyStack.
	
	aStack push: 'a'.
	self assert: aStack size = 1.
	
	aStack push: 'b'.
	self assert: aStack size = 2! !

!CollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:36'!
testSize
	
	| aStack |
	
	aStack := self emptyStack.
	
	self assert: aStack size = 0.
	
	aStack push: 'a'.
	self assert: aStack size = 1.
	
	aStack push: 'b'.
	self assert: aStack size = 2.
	
	aStack pop.
	self assert: aStack size = 1.
	
	aStack pop.
	self assert: aStack size = 0.

 
	


! !

!CollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:37'!
testTop

	| aStack |
	
	aStack := self emptyStack.

	self assert: aStack isEmpty.
	
	aStack push: 'a'.
	aStack push: 'b'.
	self assert: aStack top = 'b'.
	self assert: aStack top = 'b'.
	self assert: aStack size = 2.! !

TestCase subclass: #NonCollectionStackTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'UCA_IS2-Stack-Tests'!

!NonCollectionStackTests methodsFor: 'examples' stamp: 'Lucas 10/23/2012 13:51'!
emptyStack

	"Deberia devolver un stack vaco. -jmaestri"
	
	^(Stack new)! !

!NonCollectionStackTests methodsFor: 'examples' stamp: 'Lucas 10/23/2012 13:49'!
nonEmptyStack

	"Deberia devolver un stack no vaco. -jmaestri"
	
	|stackPila|
	stackPila:=Stack new.
	stackPila push: 'elemento'.
	^stackPila.! !

!NonCollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:28'!
testEmptyError

	| aStack |
	aStack := self emptyStack.
	self should: [ aStack top ] raise: Error.
	self should: [ aStack pop] raise: Error.
	
	aStack push: 'element'.
	
	self shouldnt: [ aStack top ] raise: Error.
	self shouldnt: [ aStack pop] raise: Error.
	
	"The stack is empty again due to previous pop"
	self should: [ aStack top ] raise: Error.
	self should: [ aStack pop] raise: Error.! !

!NonCollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:28'!
testIsEmpty

	self assert: (self emptyStack isEmpty).
	self deny: (self nonEmptyStack isEmpty).! !

!NonCollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:28'!
testPop

	| aStack res elem |

	elem := 'anElement'.
	aStack := self emptyStack.
	
	aStack push: 'a'.
	aStack push: elem.
	
	res := aStack pop.
	self assert: res = elem.
	self assert: res == elem.
	self assert: aStack size = 1.
	
	aStack pop.
	self assert: aStack isEmpty! !

!NonCollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:28'!
testPush

	| aStack |

	aStack := self emptyStack.
	
	aStack push: 'a'.
	self assert: aStack size = 1.
	
	aStack push: 'b'.
	self assert: aStack size = 2! !

!NonCollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:28'!
testSize
	
	| aStack |
	aStack := self emptyStack.
	
	self assert: aStack size = 0.
	
	aStack push: 'a'.
	self assert: aStack size = 1.
	
	aStack push: 'b'.
	self assert: aStack size = 2.
	
	aStack pop.
	self assert: aStack size = 1.
	
	aStack pop.
	self assert: aStack size = 0.

 
	


! !

!NonCollectionStackTests methodsFor: 'tests' stamp: 'jmaestri 10/16/2012 20:28'!
testTop

	| aStack |
	aStack := self emptyStack.

	self assert: aStack isEmpty.
	
	aStack push: 'a'.
	aStack push: 'b'.
	self assert: aStack top = 'b'.
	self assert: aStack top = 'b'.
	self assert: aStack size = 2.! !
